#!/bin/sh

# Copyright (c) 2009 Thinstuff s.r.o
# License: GPL Version 2
# This script will download and build openssl for iPhone (armv6) and i386

OPENSSLVERSION="0.9.8j"
MD5SUM="a5cb5f6c3d11affb387ecf7a997cac0c"
OPENSSLPATCH="OpenSSL-Thinstuff.diff"

CS=`md5 -q "openssl-$OPENSSLVERSION.tar.gz" 2>/dev/null`
if [ ! "$CS" = "$MD5SUM" ]; then
    echo "Downloading OpenSSL Version $OPENSSLVERSION ..."
    rm -f "openssl-$OPENSSLVERSION.tar.gz"
    curl -o "openssl-$OPENSSLVERSION.tar.gz" http://www.openssl.org/source/openssl-$OPENSSLVERSION.tar.gz

    CS=`md5 -q "openssl-$OPENSSLVERSION.tar.gz" 2>/dev/null`
    if [ ! "$CS" = "$MD5SUM" ]; then
	echo "Download failed or invalid checksum. Have a nice day."
	exit 1
    fi
fi

rm -rf openssltmp
mkdir openssltmp
cd openssltmp

echo "Unpacking OpenSSL ..."
tar xfz "../openssl-$OPENSSLVERSION.tar.gz"
if [ ! $? = 0 ]; then
    echo "Unpacking failed."
    exit 1
fi
echo

echo "Applying Thinstuff patch ..."

cd "openssl-$OPENSSLVERSION"
patch -p1 < "../../$OPENSSLPATCH"

if [ ! $? = 0 ]; then
    echo "Patch failed."
    exit 1
fi
echo

mkdir -p ../../openssl
rm -f ../../openssl/*.h

echo "Copying header hiles ..."
cp include/openssl/*.h ../../openssl/
echo

echo "Building i386 version (for simulator). Please wait ..."
./Configure darwin-i386-cc >BuildLog.darwin-i386.txt
make >>BuildLog.darwin-i386.txt 2>&1
echo "Done. Build log saved in BuildLog.darwin-i386.txt"
cp libcrypto.a ../../openssl/libcrypto_i386.a
cp libssl.a ../../openssl/libssl_i386.a
make clean >/dev/null 2>&1
echo

echo "Building armv6 version (for iPhone). Please wait ..."
./Configure darwin-armv6-cc >BuildLog.darwin-armv6.txt
make >>BuildLog.darwin-armv6.txt 2>&1
echo "Done. Build log saved in BuildLog.darwin-armv6.txt"
cp libcrypto.a ../../openssl/libcrypto_armv6.a
cp libssl.a ../../openssl/libssl_armv6.a
make clean >/dev/null 2>&1
echo

echo "Finished. Please verify the contens of the openssl folder in your main project folder"

